from dataclasses import dataclass
from typing import List, Dict, Optional


@dataclass
class BranchEvent:
    branches: List[str]
    types: Optional[List[str]] = None

    def to_dict(self):
        return {k: v for k, v in self.__dict__.items() if v is not None}


@dataclass
class On:
    pull_request: Optional[BranchEvent] = None
    push: Optional[BranchEvent] = None
    workflow_dispatch: Optional[bool] = False
    repository_dispatch: Optional[Dict[str, str]] = None

    def to_dict(self):
        result = {}

        if self.pull_request is not None:
            result["pull_request"] = self.pull_request.to_dict()

        if self.push is not None:
            result["push"] = self.push.to_dict()

        if self.workflow_dispatch:
            result["workflow_dispatch"] = []

        if self.repository_dispatch is not None:
            result["repository_dispatch"] = self.repository_dispatch

        return result

    def __getstate__(self):
        return self.to_dict()



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="pyciactions",
    version="1.0.11-alpha-8",
    description="Declarative builder for Github Action Scripts",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Julien Bongars",
    packages=["pyciactions", "pyciactions.github"],
    install_requires=["dataclasses==0.6", "PyYAML==6.0"],
    url="https://github.com/JBongars/pyciactions",
)



import yaml
from datetime import datetime
from . import IWorkflow


def str_presenter(dumper, data):
    if len(data.splitlines()) > 1:  # check for multiline string
        return dumper.represent_scalar("tag:yaml.org,2002:str", data, style="|")
    return dumper.represent_scalar("tag:yaml.org,2002:str", data)


yaml.add_representer(str, str_presenter)

yaml.representer.SafeRepresenter.add_representer(str, str_presenter)


def generate(workflow: IWorkflow, filename: str):
    workflow_dict = workflow.to_dict()
    yaml_string = yaml.dump(workflow_dict, sort_keys=False).replace(" |2 ", " | ")

    print(
        f"""
echo -e "# Generated automatically on {datetime.now().isoformat()} using pyactions

cat <<'EOF' > {filename}
{yaml_string}
EOF
"""
    )



from dataclasses import dataclass
from typing import Optional, List, Dict

from pyciactions import IWorkflow
from .on import On
from .job import Job


@dataclass
class Workflow(IWorkflow):
    name: str
    on: On
    jobs: List[Job]
    permissions: Optional[Dict[str, str]] = None
    env: Optional[Dict[str, str]] = None

    def to_dict(self):
        result = {
            "name": self.name,
            "on": self.on.to_dict(),
            "jobs": {j.id: j.to_dict() for j in self.jobs},
        }
        additional_attrs = {
            k: v for k, v in vars(self).items() if v is not None and k not in result
        }
        result.update(additional_attrs)
        return result

    def __getstate__(self):
        return self.to_dict()
